subroutine initialize()
!-----------------------------------------------------------------------
! Purpose: 
!          Initialize the arrays
! Inputs:  
!          None
!-----------------------------------------------------------------------

  use module_data
  implicit none
  real*8 :: r, p, xi2
  real*8 :: TwoPi
  integer :: i, j

  ! allocate variables
  allocate( disp(nFreqPoint) )
  allocate( velo(nFreqPoint) )
  allocate( accl(nFreqPoint) )
  disp = disp0
  velo = velo0

  allocate( MaxDisp(nFreqPoint) )
  allocate( MaxVelo(nFreqPoint) )
  allocate( MaxAccl(nFreqPoint) )
  MaxDisp = 0.0d0
  MaxVelo = 0.0d0
  MaxAccl = 0.0d0

  allocate( Keff(nFreqPoint) )
  allocate( Qeff(nFreqPoint) )
  Keff = 0.0d0
  Qeff = 0.0d0

  allocate( freq(nFreqPoint) )
  allocate( xi_freq(nFreqPoint) )

  ! calculate the frequencies
  TwoPi = DATAN(1.0d0) * 8.0d0
  if (.not. useLogScale) then ! if the frequencies are distributed evenly
     dFreq = (MaxFreq - MinFreq) / (nFreqPoint - 1)
     freq(1) = MinFreq
     do i = 2, nFreqPoint
        freq(i) = freq(i-1) + dFreq
     end do
     freq = freq * TwoPi

  else ! if the log scale will be used
     r = MaxFreq / MinFreq
     p = 1.0d0 / (nFreqPoint - 1)
     dFreq = r**p
     freq(1) = MinFreq
     freq(1) = freq(1) * TwoPi
     do i = 2, nFreqPoint
        freq(i) = freq(i-1) * dFreq
     end do

  end if

  ! Generate stiffness and damping
  xi2 = 2.0d0 * damp_xi
  xi_freq = freq * xi2 ! 2.0 * \xi * \omega_i
  Keff = freq * freq ! K = \omega_i^2
  accl = -Keff * disp - xi_freq * velo ! Initial acceleration

  ! set parameters for time integration
  dt = TotalTime / nStep
  nStartStep = StartTime / dt   
  dt_load = LoadTime / nLoadPoint ! The load is assumed to be zero at t = 0
  dt_load_inv = 1.0d0 / dt_load

end subroutine initialize
