subroutine ReadData()
!------------------------------------------------------------------
!	purpose: Read macro commands and their parameters
!          Allocate and initialize arrays
!
!	Input:
!            None		 
!------------------------------------------------------------------

    use module_data
    use module_output
    implicit none

    integer, parameter:: nmacro = 13
    character(4),parameter:: macros(nmacro) = (/ &
        'end ', 'time', 'nstp', 'nld ', 'fmax', &
        'fmin', 'nf  ', 'log ', 'load', 'name', &
        'curv', 'inte', 'damp'/)

    logical :: end_of_file
    integer :: i, j, istep, jstep
  
    logical :: Read_Time = .false.
    logical :: Read_nLoad = .false.
    logical :: Read_nStep = .false.
    logical :: Read_fmax = .false.
    logical :: Read_fmin = .false.
    logical :: Read_nFreq = .false.
    logical :: Read_Load = .false.
    logical :: Read_Inte = .false.
    logical :: Read_damp = .false.

    write(io_log, 100)
    write(*, 100)

    end_of_file= .false.
    do while(.not. end_of_file)

    !       write(*,101) macro
    !       write(io_log,101) macro

        select case ( KeyWord(macros, nmacro) )
        case (1) ! 'end '
            end_of_file = .true.

        case (2) ! 'time'
            StartTime = GetReal()
            if (StartTime .lt. 0) then
                write(*, *) 'WARNING: The start time of sampling must be zero or positive!'
                write(io_log, *) 'WARNING: The start time of sampling must be zero or positive!'
                write(*, *) 'WARNING: The start time is set to zero!'
                write(io_log, *) 'WARNING: The start time is set to zero!'
                StartTime = 0.0d0
            end if
            TotalTime = GetReal()
            if (TotalTime .le. 0) then
                write(*, *) 'FATAL: The total time must be positive!'
                write(io_log, *) 'FATAL: The total time must be positive!'
                stop
            end if
            Read_Time = .true.

        case (3) ! 'nstp'
           nStep = GetInt()
           if (nStep <= 0) then
              nStep = 10
              write(*, *) 'WARNING: The number of integration steps must be positive!'
              write(*, *) 'WARNING: nStep reset to 10!'
              write(io_log, *) 'WARNING: The number of integration steps must be positive!'
              write(io_log, *) 'WARNING: nStep reset to 10!'
           end if
           Read_nStep = .true.

        case (4) ! 'nld '
           nLoadPoint = GetInt()
           if (nLoadPoint .le. 0) then
              nLoadPoint = 1
              write(*, *) 'WARNING: The number of load history points must be positive!'
              write(*, *) 'WARNING: nLoadPoint reset to 1!'
              write(io_log, *) 'WARNING: The number of load history points must be positive!'
              write(io_log, *) 'WARNING: nLoadPoint reset to 1!'
           end if
           LoadTime = GetReal()
           if (LoadTime .le. 0.0d0) then
              write(*, *) 'FATAL: The loading time must be positive!'
              write(io_log, *) 'FATAL: The loading time must be positive!'
              stop
           end if
           Read_nLoad = .true.
            
        case (5) ! 'fmax' - Set maximum frequency
           MaxFreq = GetReal()
           if (MaxFreq .le. 0) then
              write(*, *) 'FATAL: The maximum frequency must be positive!'
              write(io_log, *) 'FATAL: The maximum frequency must be positive!'
              stop
           end if
           Read_fmax = .true.

        case (6) ! 'fmin' - Set minimum frequency
           MinFreq = GetReal()
           if (MinFreq .le. 0) then
              write(*, *) 'FATAL: The minimum frequency must be positive!'
              write(io_log, *) 'FATAL: The minimum frequency must be positive!'
              stop
           end if
           Read_fmin = .true.

        case (7) ! 'nf  ' - Set the number of frequencies
           nFreqPoint = GetInt()
           if (nFreqPoint .le. 1) then
              nFreqPoint = 2
              write(*, *) 'WARNING: The number of frequencies must be larger than 1!'
              write(*, *) 'WARNING: nFreqPoint reset to 2!'
              write(io_log, *) 'WARNING: The number of frequencies must be larger than 1!'
              write(io_log, *) 'WARNING: nFreqPoint reset to 2!'
           end if
           Read_nFreq = .true.

        case (8) ! 'log '
           i = GetInt()
           useLogScale = (i .ne. 0)

        case (9) ! 'load' - Read loading history
           if (Read_nLoad) then
              allocate( Load(0:nLoadPoint) )
              call ReadLoad()
		    
           else
              write(*, *) 'FATAL: Commands nld  must precede load!'
              write(io_log, *) 'FATAL: Commands nld  must precede load!'
              stop
		        
           end if
           Read_Load = .true.
		    
        case (10) ! 'name' - Read the "kernel" of the names of the output files 
           filename = GetWord()
           do i = 1, MAX_OUTPUT
              filename_spectrum(i) = trim(adjustL(filename))//"_"//trim(adjustL(filename_spectrum(i)))
           end do

        case (11) ! 'curv' - Set which curves to be output
           do i = 1, MAX_OUTPUT
              j = GetInt()
              OutCurves(i) = (j .ne. 0)
           end do

        case (12) ! 'inte' - Set which integration method to be used. Currently only Newmark method can be used
           InteType = GetInt()
           if (InteType .ne. 1) then
              InteType = 1
              write(*, *) 'WARNING: Only Newmark method can be used for time integration'
              write(io_log, *) 'WARNING: Only Newmark method can be used for time integration'
           end if
           IntParas(1) = GetReal()
           IntParas(2) = GetReal()
           Read_Inte = .true.

        case (13) ! 'damp' - Set damping ratio
           damp_xi = GetReal()
           if (damp_xi .lt. 0) then
              damp_xi = dabs(damp_xi)
              write(*, *) 'WARNING: The damping ratio must be positive!'
              write(*, *) 'WARNING: The absolute value of damping ratio is used!'
              write(io_log, *) 'WARNING: The damping ratio must be positive!'
              write(io_log, *) 'WARNING: The absolute value of damping ratio is used!'
           end if
           Read_damp = .true.

        case default
           write(*,102) 
           write(io_log,102) 
		
        end select

     end do ! while(.not. end_of_file)

    close(io_in)
	write(*,103)	  
	write(io_log,103)	  

    if (.not. Read_Time) then
        write(*, *) 'FATAL: The total time must be set!'
        write(io_log, *) 'FATAL: The total time must be set!'
        stop
    end if
    if (.not. Read_nLoad) then
        write(*, *) 'FATAL: The number of load history points must be set!'
        write(io_log, *) 'FATAL: The number of load history points must be set!'
        stop
    end if
    if (.not. Read_nStep) then
        write(*, *) 'WARNING: The number of total simulation steps must be set!'
        write(*, *) 'WARNING: nStep is set to be 11!'
        write(io_log, *) 'WARNING: The number of total simulation steps must be set!'
        write(io_log, *) 'WARNING: nStep is set to be 11!'
    end if
    if (.not. Read_fmax .or. .not. Read_fmin) then
        write(*, *) 'FATAL: The upper and the lower limit of frequencies must be set!'
        write(io_log, *) 'FATAL: The upper and the lower limit of frequencies must be set!'
        stop
    end if
    if (.not. Read_nFreq) then
        write(*, *) 'WARNING: The number of frequencies must be set!'
        write(*, *) 'WARNING: nFreqPoint is set to be 11'
        write(io_log, *) 'WARNING: The number of frequencies must be set!'
        write(io_log, *) 'WARNING: nFreqPoint is set to be 11'
    end if
    if (.not. Read_Load) then
        write(*, *) 'FATAL: The load history must be set!'
        write(io_log, *) 'FATAL: The load history must be set!'
        stop
    end if
    if (.not. Read_Inte) then
       IntParas(1) = 0.5d0
       IntParas(2) = 0.25d0
    end if
    if (.not. Read_damp) then
        write(*, *) 'FATAL: The damping ratio must be set!'
        write(io_log, *) 'FATAL: The damping ratio must be set!'
        stop
    end if
    
    close(io_in)

100   format(/, '======Begin to read marco commands and parameters======')
102   format(/,'***Error : It is a wrong macro command ! Check input file!')
103   format(/, '------End of reading marco commands and parameters------')

end subroutine ReadData

! ---------------------------------------------------------------------
subroutine ReadLoad()
!------------------------------------------------------------------
!	purpose: 
!                Read the load history points
!	Input:
!                None		 
!------------------------------------------------------------------

    use module_data
    use module_output
    implicit none
    logical :: exist_infile
    integer :: i
    
    filename_load = GetWord()
    LoadType = GetInt()
    if (LoadType .ne. 2) then
       LoadType = 1
    end if

    INQUIRE(FILE = trim(adjustL(filename_load)), EXIST = exist_infile)
    IF (.NOT. exist_infile) THEN
        write(*, *) "FATAL: LOAD FILE DOES NOT EXIST !"
        write(io_log, *)  "FATAL: LOAD FILE DOES NOT EXIST !"
        STOP
    END IF

    open(unit=io_load, file=trim(adjustL(filename_load)), status='old')
    
    Load(0) = 0.0d0
    read(io_load, *) (Load(i), i = 1, nLoadPoint)

    if (LoadType == 1) then
       Load = -Load
    end if

    close(io_load)
    
end subroutine ReadLoad
