subroutine TIM_Newmark()

  use module_data
  use FFI
  implicit none

  integer :: i, j, icurr
  real*8 :: c(0:7), gamma, beta, t, dt2, f
  real*8, allocatable :: dtemp(:), atemp(:)
  integer :: istep, jstep

  if(IntParas(1) .eq. 0.0) then
     IntParas(1) = 0.25d0
  end if
  if(IntParas(2) .eq. 0.0) then
     IntParas(2) = 0.5d0
  end if
  write(*, 301) IntParas(1), IntParas(2)
  write(io_log, 301) IntParas(1), IntParas(2)
  
  beta = IntParas(1)
  gamma = IntParas(2)
  dt2 = dt * dt
  c(0) = 1.0d0 / (beta*dt2)
  c(1) = gamma / beta / dt 
  c(2) = 1.0d0 / beta / dt 
  c(3) = 0.5d0/beta - 1.0d0
  c(4) = gamma/beta - 1.0d0 
  c(5) = dt*(gamma/2.0d0/beta - 1.0d0) 
  c(6) = dt*(1.0d0 - gamma)
  c(7) = gamma*dt

  ! Calculate the effective stiffness matrix, and store its inverse
  Keff = Keff + c(0) * 1.0d0 + c(1) * xi_freq
  Keff = 1.0d0 / Keff

  t = 0.0d0
  MaxDisp = dabs(disp)
  MaxVelo = dabs(velo)
  MaxAccl = dabs(accl)
  allocate( dtemp(nFreqPoint) )
  allocate( atemp(nFreqPoint) )

  ! Time stepping
  do i = 1, nStep
     t = t + dt
     call force(t, f)

     ! Q_eff = f + (M_matrix .x. (c0*d1 + c2*v1 +c3*a1)) + (C_matrix .x. (c1*d1 + c4*v1 +c5*a1))
     Qeff = f + (c(0)*disp + c(2)*velo + c(3)*accl) + &
          & xi_freq * (c(1)*disp + c(4)*velo + c(5)*accl) 

     ! Calculate displacements, velocities and accelerations at t^{n+1}
     dtemp = disp
     disp = Keff * Qeff
     ! a2=c0*(d2-d1) - c2*v1 - c3*a1; v2=v1 + c6*a1 +c7*a2
     atemp = accl
     accl = c(0)*(disp - dtemp) - c(2)*velo - c(3)*atemp
     velo = velo + c(6)*atemp + c(7)*accl
        
     ! Record maximum value
     if (t .ge. StartTime) then
        forall( j = 1: nFreqPoint )
           MaxDisp(j) = max( MaxDisp(j), dabs(disp(j)) )
           MaxVelo(j) = max( MaxVelo(j), dabs(velo(j)) )
           MaxAccl(j) = max( MaxAccl(j), dabs(accl(j)) )
        end forall
     end if
  end do

  deallocate( dtemp )
  deallocate( atemp )

301 format(4X,'beta = ',f7.4,8X,'gamma = ',f7.4)

end subroutine TIM_Newmark
